@@ -12,7 +12,7 @@ use Sys::Syslog qw(:macros);
 use LISM::Storage;
 use Data::Dumper;
 
-our $VERSION = '2.1.3';
+our $VERSION = '2.1.5';
 
 our $lism_master = 'lism_master';
 our $syncrdn = 'cn=sync';
@@ -741,6 +741,7 @@ sub _do_search
     }
 
     foreach my $srchbase (@srchbases) {
+        my $dfilterStr = $filterStr;
         my @subentries;
 
         my $dname = $self->_getDataName($srchbase);
@@ -756,13 +757,13 @@ sub _do_search
         }
 
         # do pre handler
-        $rc = $self->_doHandler('pre', 'search', $dname, \$srchbase, \$filterStr);
+        $rc = $self->_doHandler('pre', 'search', $dname, \$srchbase, \$dfilterStr);
         if ($rc) {
             last;
         }
 
         # call search of the appropriate storage
-        ($rc, @subentries) = $storage->search($srchbase, $scope, $deref, $sizeLim, $timeLim, $filterStr, $attrOnly, @attrs);
+        ($rc, @subentries) = $storage->search($srchbase, $scope, $deref, $sizeLim, $timeLim, $dfilterStr, $attrOnly, @attrs);
         if ($rc == LDAP_SERVER_DOWN) {
             $self->log(level => 'error', message => "Searching in $dname failed($rc)");
             $rc = LDAP_SUCCESS;
@@ -1121,7 +1122,7 @@ sub _doSync
             if (defined($sobject->{syncattrs})) {
                 @sync_attrs = @{$sobject->{syncattrs}};
             } else {
-                @sync_attrs = ($info[0] =~ /^([^:]+):/gmi);
+                @sync_attrs = $self->_unique($info[0] =~ /^([^:]+):/gmi);
             }
 
             for (my $j = 0; $j < @sync_attrs; $j++) {
@@ -1320,7 +1321,7 @@ sub _getSyncInfo
                     foreach my $op (@{$sdata->{masterop}}) {
                         $ops{$op} = 1;
                         if ($op eq 'delete') {
-                            $deletedn{$sbase} = 1;
+                            $deletedn{$sbase} = [$dname, $oname];
                         }
                     }
 
@@ -1346,7 +1347,7 @@ sub _getSyncInfo
                     # comare data storage's values with master one
                     for (my $i = 0; $i < @entries; $i++) {
                         my ($dn) = ($entries[$i] =~ /^dn: (.*)\n/);
-                        my ($key) = ($dn =~ /^(.*),$dbase$/);
+                        my ($key) = ($dn =~ /^(.*),$dbase$/i);
                         if (!$key) {
                             next;
                         }
@@ -1392,6 +1393,7 @@ sub _getSyncInfo
                                     next;
                                 }
 
+                                $pvals =~ s/([.*+?\[\]])/\\$1/g;
                                 if ($dvals !~ /^$pvals$/i && $ops{modify}) {
                                     $nosync_data{$dname} = 1;
                                     $nosync_entries{lc($dn)} = 1;
@@ -1410,8 +1412,20 @@ sub _getSyncInfo
 
         # check deleted entry in present list
         foreach my $sbase (keys %deletedn) {
+            my $sobject = $sync->{data}{$deletedn{$sbase}[0]}->{object}{$deletedn{$sbase}[1]};
+
+            # synchronization filter
+            my $masterfilter = undef;
+            if (defined($sobject->{masterfilter})) {
+                $masterfilter = Net::LDAP::Filter->new($sobject->{masterfilter}[0]);
+            }
+
             foreach my $key (keys %{$present_list->{$sbase}}) {
                 if (!$present_list->{$sbase}{$key}->{present}) {
+                    if (defined($masterfilter) &&
+                        !LISM::Storage->parseFilter($masterfilter, $present_list->{$sbase}{$key}->{entryStr})) {
+                        next;
+                    }
                     $nosync_entries{lc("$key,$sbase")} = 1;
                 }
             }
@@ -1465,7 +1479,7 @@ sub _getSyncInfo
                     # comare data storage's values with master one
                     for (my $i = 0; $i < @entries; $i++) {
                         my ($dn) = ($entries[$i] =~ /^dn: (.*)\n/);
-                        my ($key) = ($dn =~ /^(.*),$dbase$/);
+                        my ($key) = ($dn =~ /^(.*),$dbase$/i);
                         if (!$key) {
                             next;
                         }
@@ -1510,6 +1524,7 @@ sub _getSyncInfo
                                     next;
                                 }
 
+                                $pvals =~ s/([.*+?\[\]])/\\$1/g;
                                 if ($dvals !~ /^$pvals$/i && $ops{modify}) {
                                     $nosync_data{$dname} = 1;
                                     $nosync_entries{lc($dn)} = 1;
@@ -1617,7 +1632,7 @@ sub _setSyncInfo
                     foreach my $op (@{$sdata->{masterop}}) {
                         $ops{$op} = 1;
                         if ($op eq 'delete') {
-                            $deletedn{$sbase} = 1;
+                            $deletedn{$sbase} = [$dname, $oname];
                         }
                     }
 
@@ -1648,7 +1663,7 @@ sub _setSyncInfo
                             next;
                         }
 
-                        my ($key) = ($entries[$i] =~ /^dn: (.*),$dbase\n/);
+                        my ($key) = ($entries[$i] =~ /^dn: (.*),$dbase\n/i);
                         if (!$key) {
                             next;
                         }
@@ -1733,6 +1748,7 @@ sub _setSyncInfo
                                     next;
                                 }
 
+                                $pvals =~ s/([.*+?\[\]])/\\$1/g;
                                 if ($dvals !~ m/^$pvals$/i && $ops{modify}) {
                                     if (!$pocs) {
                                         # Add objectClass for adding new attribute
@@ -1754,16 +1770,19 @@ sub _setSyncInfo
                                     }
 
                                     if (@sync_vals) {
+                                        my @mod_vals;
                                         for (my $k = 0; $k < @sync_vals; $k++) {
-                                            $sync_vals[$k] = encode('utf8', $sync_vals[$k]);
+                                            $mod_vals[$k] = encode('utf8', $sync_vals[$k]);
                                         }
-                                        push(@modlist, ('REPLACE', $attr, @sync_vals));
+                                        push(@modlist, ('REPLACE', $attr, @mod_vals));
                                     } else {
                                         push(@modlist, ('DELETE', $attr));
                                     }
 
                                     $entryStr =~ s/$attr: .*\n//gi;
-                                    $entryStr = "$entryStr$dvals\n";
+                                    foreach my $value (@sync_vals) {
+                                        $entryStr = "$entryStr\n$attr: $value";
+                                    }
                                 }
 		            }
 
@@ -1791,8 +1810,21 @@ sub _setSyncInfo
 
         # delete entries which don't exist in data storages
         foreach my $sbase (keys %deletedn) {
+            my $sobject = $sync->{data}{$deletedn{$sbase}[0]}->{object}{$deletedn{$sbase}[1]};
+
+            # synchronization filter
+            my $masterfilter = undef;
+            if (defined($sobject->{masterfilter})) {
+                $masterfilter = Net::LDAP::Filter->new($sobject->{masterfilter}[0]);
+            }
+
             foreach my $key (keys %{$present_list->{$sbase}}) {
                 if (!$present_list->{$sbase}{$key}->{present}) {
+                    if (defined($masterfilter) &&
+                        !LISM::Storage->parseFilter($masterfilter, $present_list->{$sbase}{$key}->{entryStr})) {
+                        next;
+                    }
+
                     my $nosync_entry = "$key,$sbase";
 
                     $nosync_entry = encode('utf8', $nosync_entry);
@@ -1833,9 +1865,6 @@ sub _setSyncInfo
 
                 foreach my $op (@{$sdata->{syncop}}) {
                     $ops{$op} = 1;
-                    if ($op eq 'delete') {
-                        $deletedn{$sobject->{syncdn}[0]} = 1;
-                    }
                 }
 
                 foreach my $syncdn (@{$sobject->{syncdn}}) {
@@ -1864,7 +1893,7 @@ sub _setSyncInfo
                     # comare data storage's values with master one
                     for (my $i = 0; $i < @entries; $i++) {
                         my ($dn) = ($entries[$i] =~ /^dn: (.*)\n/);
-                        my ($key) = ($dn =~ /^(.*),$dbase$/);
+                        my ($key) = ($dn =~ /^(.*),$dbase$/i);
                         if (!$key) {
                             next;
                         }
@@ -1913,16 +1942,18 @@ sub _setSyncInfo
                                     next;
                                 }
 
+                                $pvals =~ s/([.*+?\[\]])/\\$1/g;
                                 if ($dvals !~ m/^$pvals$/i) {
                                     if (@{$left_vals}) {
                                         push(@sync_vals, @{$left_vals});
                                     }
 
                                     if (@sync_vals) {
+                                        my @mod_vals;
                                         for (my $k = 0; $k < @sync_vals; $k++) {
-                                            $sync_vals[$k] = encode('utf8', $sync_vals[$k]);
+                                            $mod_vals[$k] = encode('utf8', $sync_vals[$k]);
                                         }
-                                        push(@modlist, ('REPLACE', $attr, @sync_vals));
+                                        push(@modlist, ('REPLACE', $attr, @mod_vals));
                                     } else {
                                         push(@modlist, ('DELETE', $attr));
                                     }
