@@ -5,6 +5,7 @@ use base qw(LISM::Handler);
 use Net::LDAP;
 use Net::LDAP::Constant qw(:all);
 use URI;
+use Encode;
 use Sys::Syslog;
 use Sys::Syslog qw(:macros);
 use Data::Dumper;
@@ -109,6 +110,10 @@ sub pre_search
 
     foreach my $rule (@{$conf->{rewrite}}) {
         if ($rule->{context} eq 'request') {
+            if (defined($rule->{dn}) && ${$basep} !~ /$rule->{dn}/i) {
+                next;
+            }
+
             my %rwcache;
             my $substitution = $rule->{substitution};
 
@@ -149,10 +154,11 @@ sub post_search
             for (my $i = 0; $i < @{$entriesp}; $i++) {
                 my %rwcache;
                 my $rc = 0;
+                my $entryStr = encode('utf8', ${$entriesp}[$i]);
                 my $substitution = $rule->{substitution};
-                $substitution =~ s/%0/${$entriesp}[$i]/;
+                $substitution =~ s/%0/$entryStr/;
 
-                my (@line) = split(/\n/, ${$entriesp}[$i]);
+                my (@line) = split(/\n/, $entryStr);
                 my ($dn) = ($line[0] =~ /^dn: (.*)$/);
                 if (defined($rule->{dn}) && $dn !~ /$rule->{dn}/i) {
                     next;
@@ -177,7 +183,7 @@ sub post_search
                     }
                 }
                 if (!$rc) {
-                    ${$entriesp}[$i] = "$line[0]\n".join("\n", $self->_unique(@replaced))."\n";
+                    ${$entriesp}[$i] = decode('utf8', "$line[0]\n".join("\n", $self->_unique(@replaced))."\n");
                 }
             }
         }
@@ -442,14 +448,18 @@ sub _rewriteParse
     if ($str =~ /$match/i) {
         # do functions
         my @subs = ($substitution);
-        my @rwmaps = ($substitution =~ /%{([^}]*)}/g);
+        my @rwmaps = ($substitution =~ /%{([^(]*\([^)]*\).*)}/g);
         foreach my $rwmap (@rwmaps) {
             my @values;
             my $key = lc($rwmap);
             if (defined(${$rwcache}{$key})) {
                 @values = @{${$rwcache}{$key}};
             } else {
-                my ($map_name, $map_args) = ($rwmap =~ /^([^(]*)\((.*)\)$/);
+                my ($map_name, $map_args) = ($rwmap =~ /^([^(]*)\((.*)\)$/s);
+                if (!$map_name) {
+                    return undef;
+                }
+
                 @values = $self->_rewriteMap($map_name, $map_args);
                 if (!defined(@values)) {
                     return undef;
@@ -518,6 +528,7 @@ sub _ldapMap
         $ldapmap->{ldap} = Net::LDAP->new($ldapmap->{uri});
         if (!defined($ldapmap->{ldap})) {
             $self->log(level => 'error', message => "Can't connect $ldapmap->{uri}");
+            return undef;
         }
 
         $msg = $ldapmap->{ldap}->bind($ldapmap->{binddn}, password => $ldapmap->{bindpw});
