@@ -64,6 +64,29 @@ sub pre_add
     return 0;
 }
 
+sub _checkConfig
+{
+    my $self = shift;
+    my $conf = $self->{_config};
+    my $rc = 0;
+
+    if ($rc = $self->SUPER::_checkConfig()) {
+        return $rc;
+    }
+
+    if (defined($conf->{libload})) {
+        foreach my $lib (@{$conf->{libload}}) {
+            eval "do \'$lib\'";
+            if ($@) {
+                $self->log(level => 'alert', message => "setval do require $lib: $@");
+                return 1;
+            }
+        }
+    }
+
+    return $rc;
+}
+
 sub _getStaticValue
 {
     my $self = shift;
@@ -74,6 +97,9 @@ sub _getStaticValue
     if (defined($static->{value})) {
         if ($static->{value}[0]->{type} eq 'function') {
             eval "\$value = $static->{value}[0]->{content}";
+            if ($@) {
+                $self->log(level => 'error', message => "setval $static->{value}[0]->{content} failed: $@");
+            }
         } else {
             $value = $static->{value}[0]->{content};
         }
