@@ -10,6 +10,8 @@ use Sys::Syslog;
 use Sys::Syslog qw(:macros);
 use Data::Dumper;
 
+our $rawattrs = '^(jpeg|photo|objectSid|objectGUID|.*;binary)$';
+
 =head1 NAME
 
 LISM::Storage::LDAP - LDAP storage for LISM
@@ -33,7 +35,12 @@ sub init
 
     $self->SUPER::init();
 
-    ($conf->{nc}) = ($conf->{uri}[0] =~ /^ldaps?:\/\/[^\/]+\/(.+)$/i);
+    if (!defined($conf->{nc})) {
+        if (utf8::is_utf8($conf->{uri}[0])) {
+            $conf->{uri}[0] = encode('utf8', $conf->{uri}[0]);
+        }
+        ($conf->{nc}) = ($conf->{uri}[0] =~ /^ldaps?:\/\/[^\/]+\/(.+)$/i);
+    }
 
     return 0;
 }
@@ -132,9 +139,9 @@ sub search
     # Attribute mapping
     foreach my $ldapmap (@{$conf->{ldapmap}}) {
         if ($ldapmap->{type} =~ /^objectclass$/i) {
-            $filterStr =~ s/objectClass=$ldapmap->{local}[0]$/objectClass=$ldapmap->{foreign}[0]/mi;
+            $filterStr =~ s/objectClass=$ldapmap->{local}$/objectClass=$ldapmap->{foreign}/mi;
         } else {
-            $filterStr =~ s/\($ldapmap->{local}[0]=/\($ldapmap->{foreign}[0]=/mi;
+            $filterStr =~ s/\($ldapmap->{local}=/\($ldapmap->{foreign}=/mi;
         }
     }
 
@@ -156,18 +163,27 @@ sub search
             } else {
                 foreach my $attr ($entry->attributes) {
                     foreach my $value ($entry->get_value($attr)) {
-                        $value =~ s/$conf->{nc}$/$self->{suffix}/i;
-                        $entryStr = $entryStr."$attr: $value\n";
+                        if ($attr =~ /$rawattrs/i) {
+                            $value = encode_base64($value);
+                            $entryStr = $entryStr.$attr.":: $value";
+                        } else {
+                            $value =~ s/$conf->{nc}$/$self->{suffix}/i;
+                            $entryStr = $entryStr.$attr.": $value\n";
+                        }
                     }
                 }
             }
 
+            if (!$self->_checkEntry($entryStr)) {
+                next;
+            }
+
             # Attribute mapping
             foreach my $ldapmap (@{$conf->{ldapmap}}) {
                 if ($ldapmap->{type} =~ /^objectclass$/i) {
-                    $entryStr =~ s/^objectClass: $ldapmap->{foreign}[0]$/objectClass: $ldapmap->{local}[0]/mi;
+                    $entryStr =~ s/^objectClass: $ldapmap->{foreign}$/objectClass: $ldapmap->{local}/mi;
                 } else {
-                    $entryStr =~ s/^$ldapmap->{foreign}[0]:/$ldapmap->{local}[0]:/mi;
+                    $entryStr =~ s/^$ldapmap->{foreign}:/$ldapmap->{local}:/mi;
                 }
             }
 
@@ -209,11 +225,11 @@ sub compare
     foreach my $ldapmap (@{$conf->{ldapmap}}) {
         if ($ldapmap->{type} =~ /^objectclass$/i) {
             if ($key =~ /^objectClass$/i) {
-                $avaStr =~ s/^$ldapmap->{local}[0]$/$ldapmap->{foreign}[0]/i;
+                $avaStr =~ s/^$ldapmap->{local}$/$ldapmap->{foreign}/i;
             }
         } else {
-            if ($key =~ /^$ldapmap->{local}[0]$/i) {
-                $key = $ldapmap->{foreign}[0];
+            if ($key =~ /^$ldapmap->{local}$/i) {
+                $key = $ldapmap->{foreign};
             }
         }
     }
@@ -265,12 +281,12 @@ sub modify
             if ($ldapmap->{type} =~ /^objectclass$/i) {
                 if ($key =~ /^objectClass$/i) {
                     for (my $i = 0; $i < @values; $i++) {
-                        $values[$i] =~ s/^$ldapmap->{local}[0]$/$ldapmap->{foreign}[0]/i;
+                        $values[$i] =~ s/^$ldapmap->{local}$/$ldapmap->{foreign}/i;
                     }
                 }
             } else {
-                if ($key =~ /^$ldapmap->{local}[0]$/i) {
-                    $key = $ldapmap->{foreign}[0];
+                if ($key =~ /^$ldapmap->{local}$/i) {
+                    $key = $ldapmap->{foreign};
                 }
             }
         }
@@ -334,11 +350,11 @@ sub add
         foreach my $ldapmap (@{$conf->{ldapmap}}) {
             if ($ldapmap->{type} =~ /^objectclass$/i) {
                 if ($key =~ /^objectClass$/i) {
-                    $val =~ s/^$ldapmap->{local}[0]$/$ldapmap->{foreign}[0]/i;
+                    $val =~ s/^$ldapmap->{local}$/$ldapmap->{foreign}/i;
                 }
             } else {
-                if ($key =~ /^$ldapmap->{local}[0]$/i) {
-                    $key = $ldapmap->{foreign}[0];
+                if ($key =~ /^$ldapmap->{local}$/i) {
+                    $key = $ldapmap->{foreign};
                 }
             }
         }
@@ -453,7 +469,12 @@ sub _getConnect
     my $conf = $self->{_config};
 
     if (defined($self->{ldap}) && defined($self->{bind})) {
-        return 0;
+        my $msg = $self->{ldap}->bind($conf->{binddn}[0], password => $conf->{bindpw}[0]);
+        if ($msg->code) {
+            $self->log(level => 'error', message => "Connection check failed");
+        } else {
+            return 0;
+        }
     }
 
     $self->{ldap} = Net::LDAP->new($conf->{uri}[0]);
@@ -506,6 +527,18 @@ sub _checkConfig
     return $rc;
 }
 
+sub _checkEntry
+{
+    my $self = shift;
+    my ($entryStr) = @_;
+
+    if ($entryStr !~ /^dn: .+\n.+/) {
+        return 0;
+    }
+
+    return 1;
+}
+
 =head1 SEE ALSO
 
 L<LISM>,
