@@ -39,4 +39,11 @@ sub time2date
     return strftime("%Y%m%d%H%M%S", localtime($time))."Z";
 }
 
+sub getValue
+{
+    my ($entryStr, $attr) = @_;
+
+    return ($entryStr =~ /^$attr:\:? (.*)$/mi)[0];
+}
+
 1;
